/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.util;

import com.sun.jersey.api.client.ClientResponse;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public abstract class CreateApiTestTemplate extends BaseTestTemplate {
    private final TestPayloadReader reader = new TestPayloadReader();

    public CreateApiTestTemplate(RestDescriptor deleteDescriptor, RestDescriptor createDescriptor) {
        super(createDescriptor);
        this.deleteDescriptor = deleteDescriptor;
    }

    private String id = "createtest" + System.currentTimeMillis();
    private RestDescriptor deleteDescriptor;

    @Override
    protected String getId() {
        return id;
    }

    @Override
    protected void deleteResource() throws Exception {
        deleteDescriptor.run(getId(), testUtils.getOpsAccessToken());
    }

    @Override
    protected void createResource() {
        //do nothing
    }

    @Override
    protected int expectedOkResponseCode() {
        return 201;
    }

    @Test
    public void should_return40XResponseCode_when_makingRequest_withInvalidPayload() throws Exception {
        ClientResponse response = descriptor.runWithCustomPayload(getId(), reader.getCreateRequestPayload("createDdmsApi/invalidCreateDdmsRequestBody.json", getId(), Config.Instance().OsduTenant), testUtils.getOpsAccessToken());
        assertEquals(400, response.getStatus());
        deleteResource();
    }

    @Test
    public void should_onlyCreateMax1Resource_and_return409ForTheRest_when_creatingTheSameResourceInParallel() throws Exception {
        ExecutorService executor = Executors.newFixedThreadPool(8);
        List<Callable<ClientResponse>> tasks = new ArrayList<>();

        for (int i = 0; i < 8; i++) {
            Callable<ClientResponse> task = () -> {
                try {
                    return descriptor.run(getId(), testUtils.getOpsAccessToken());
                } catch (Exception ex) {
                    return null;
                }
            };
            tasks.add(task);
        }

        List<Future<ClientResponse>> responses = executor.invokeAll(tasks);
        executor.shutdown();
        executor.awaitTermination(20, TimeUnit.SECONDS);

        int sucessResponseCount = 0;
        int non409ErrorResponseCount = 0;
        for (Future<ClientResponse> future : responses) {
            if (future.get().getStatus() == expectedOkResponseCode())
                sucessResponseCount++;
            else if (future.get().getStatus() != 409)
                non409ErrorResponseCount++;
        }

        deleteResource();

        assertTrue(error("Expected 1 successful response. Actual " + sucessResponseCount), sucessResponseCount <= 1);
        assertEquals(error("Unexpected error response returned"), 0, non409ErrorResponseCount);
    }
}
